--[[Copyright (c) 1998-2014 Mihail Stoyanov (deather (at) gmail (dot) com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
]]

-- Name: iDKP
-- Developed by: Deatherka aka Screamerka, Al'Akir EU
-- Thanks to: Sanctusvirgo, LaughingSkullEU

local DEBUG = false

iDKP = LibStub("AceAddon-3.0"):NewAddon("iDKP", "AceEvent-3.0", "AceHook-3.0")
iDKP.title = "iDKP"
iDKP.version = "1.0.1"
iDKP.loc = LibStub("AceLocale-3.0"):GetLocale("iDKP")
iDKP.hooks = false


function iDKP:Error(msg)
	DEFAULT_CHAT_FRAME:AddMessage("iDKP: "..msg, 1.0, 0.3, 0.3);
end

function iDKP:Message(msg)
	DEFAULT_CHAT_FRAME:AddMessage("iDKP: "..msg, 1.0, 1.0, 0.5);
end

function iDKP:Debug(msg)
	if DEBUG then
		DEFAULT_CHAT_FRAME:AddMessage("iDKP: "..msg, 1.0, 1.0, 0.5);
	end
end

function zeroDKP()
	return 0,-1000,0,0
end


function iDKP:OnInitialize()
	--self:InitHooks()
end

function iDKP:OnEnable()
	self:RegisterEvent("Links_LinkOpened")
	self:InitHooks()
end

function iDKP:OnDisable()
	self:UnregisterEvent("Links_LinkOpened")
end

function iDKP:InitHooks()
	if iDKP.hooks then return end
	
	iDKP.hooks = true
	for _, tooltip in ipairs(iDKP_HookTooltips) do
		if getglobal(tooltip) then
			for _, methodName in ipairs(iDKP_HookMethods) do
				hooksecurefunc(getglobal(tooltip), methodName, self.TooltipHook)
			end
		else
			self:Error(iDKP.loc["Unable to add hooks to tooltip"].." "..tooltip)
		end
	end
end

function iDKP.TooltipHook(tooltip)
	local result,ilevel,dkp = iDKP:Calculate(nil, tooltip)
	if not result or result == 0 then return end
	
	local price = string.format("%.2f", dkp)
	if result == 2 then
		tooltip:AddDoubleLine(iDKP.loc["DKP Fixed"], price .. " ", 0.0, 1.0, 0.0, 0.0, 1.0, 0.0)
	else
		tooltip:AddDoubleLine(iDKP.loc["DKP Value"], price .. " ", 0.0, 1.0, 0.0, 0.0, 1.0, 0.0)
	end
	tooltip:Show()
end

function iDKP:Links_LinkOpened(item, tooltip)
	return self:Calculate(item, tooltip)
end

function iDKP:GetDKP(item, tooltip)
	self:Calculate(item, tooltip)
end

function iDKP:Calculate(item, tooltip)
	if not item and tooltip then
		_,item = tooltip:GetItem()
	end
		
	if not item then return zeroDKP() end
	
	local name,_,rarity,ilevel,_,_,sub_type,_,slot = GetItemInfo(item)
	local base = 0
	local coef = 0
	
	-- Static items
	if iDKP_Static[name] then
		return ilevel,iDKP_List[name]
	end
	
	if (not rarity or rarity < 2) then return zeroDKP() end
	
	if (ilevel >= 550) then -- Warlord of Draenor
		base = (ilevel - 550) / 2.6
	end
	
	-- Get slot coefficient from item slot 
	if (iDKP_Slots[slot] ~= nil) then
		coef = iDKP_Slots[slot]
		if (iDKP_SubTypes[sub_type] ~= nil) then -- Get slot coefficient from slot type
			coef = iDKP_SubTypes[sub_type]
			self:Debug("Found subtype "..sub_type.." ("..(coef)..")")
		else
			self:Debug("Found slot "..slot.." ("..(coef)..")")
		end
	else
		if string.len(slot) > 0 then
			self:Error(self.loc["Unhandled slot type"].." "..slot, 1.0, 0.7, 0.7);
		end
		return zeroDKP()
	end
	
	local dkp = base * coef;
	if dkp == 0 then return zeroDKP() end
	
	-- Process tooltip if available to find additional sockets and stats that are not tied to the item level
	if (tooltip ~= nil) then
		for i = 1, 20 do
			local ltext = getglobal(tooltip:GetName().."TextLeft"..i):GetText();
			local rtext = getglobal(tooltip:GetName().."TextRight"..i):GetText();
			if (ltext == nil) then
				break
			end
			
			for attr,val in pairs(iDKP_Attribs) do
				if (strfind(ltext, attr) ~= nil) then
					local _,_, stat = strfind(ltext, attr);
					if (stat ~= nil) then
						dkp = dkp + stat * val
						self:Debug("Found stat "..stat.." ("..(stat * val)..")")
					else
						dkp = dkp + val
						self:Debug("Found attribute "..attr.." ("..val..")")
					end
				end
			end
		end
	end
	
	return 1,ilevel,dkp,item
end